#!/usr/bin/perl
use lib '/home/erealms/ethereal/mgmt/perl';

################################################################################
# Created       : Martin Foster
# Modified      : 22-Apr-2006
################################################################################
#
# Mail - Script part of Ethereal Realms designed to compose, retreive, view
#        and manage mail
# Copyright (C) 2000-2006  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 3456 Wolfe Cres
#                 Halifax, Nova Scotia
#                 B3L 3S2
#
#################################################################################

use CGI;							# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler
use Ethereal::Filter;						# Filter handling
use Ethereal::Login;						# Login functionality
use Ethereal::Mail;						# Mail handler
use Ethereal::Param;						# Parameter control
use Ethereal::Template;						# Template handling

#################################################################################
# Gobal override
#################################################################################
$CGI::POST_MAX=1024 * 50;					# Maximum posts
$CGI::DISABLE_UPLOADS = 1;					# Disable uploads

################################################################################
# Data Members
################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter handle
my $tmpl;							# Template handler

my %mail;							# Mail hash


################################################################################
# Program Area
################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();

	# Connect and fetch
	$database->Connect($cgi);

	# Parameter handling
	$param = new Ethereal::Param($database, $cgi);
	$param->GetParam();

	# Mail hash
	$database->GetHashMail(\%mail);

	# Template instance
	$tmpl = new Ethereal::Template(\%mail);


	# Login handle
	$login = new Ethereal::Login($database, $cgi, $param);

	# Authenticate user
	if ($login->GetVerificationNormal())
	{
		# Document header
		print $database->DocumentGetHeader(), "\n";

		# Decisions
		# Remove mail
		if ((defined($param->{$mail{'TxtMailDelete'}}))
		 && (defined($param->{'MID'})))
		{ MailPurge($database, $cgi, $param, $tmpl, \%mail); }

		# Sender
		elsif (defined($param->{'SEND'}))
		{ MailSender($database, $cgi, $param, $tmpl, \%mail); }
		
		# Viewer
		elsif (defined($param->{'MID'}))
		{ MailViewer($database, $cgi, $param, $tmpl, \%mail); }
		
		# Composer
		elsif (defined($param->{'WRITE'}))
		{ MailComposer($database, $cgi, $param, $tmpl, \%mail); }

		# Sent Mail
		elsif (defined($param->{'ARCHIVE'}))
		{ MailOutbox($database, $cgi, $param, $tmpl, \%mail); }

		# Mail listing
		else
		{ MailInbox($database, $cgi, $param, $tmpl, \%mail); }


		# Document footer
		print $database->DocumentGetFooter(), "\n";
	}

#################################################################################
# Sub-Routines
################################################################################

#####################
# MailComposer
#
# Allows for the composition of mail to be sent out to the designated person.
# Since the list of puppets can get long, this will requires a textfield.

sub MailComposer
{
	#####################
	# Data members

	my $database = shift;					# Database Handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $tmpl     = shift;					# Template management
	my $mail     = shift;					# Mail hash

	my $statement;						# Database statement;
	my $res;						# My results

	my $body;						# Body of message
	my $from;						# Origin of message
	my $rcpt;						# Intended audience
	my $subj;						# Subject of message
	my $time;						# Time of posting
	my $orig;						# Original message

	my $inline;						# Inline parameter
	my $url;						# Self reference

	my $list     = '';					# List of the vain
	my $vanity   = '';					# Completed vanity template
	
	my $wchars;						# Character widget
	my $wmrcpt;						# Recipient textbox
	my $wmsubj;						# Subject box
	my $wmbody;						# Message body
	my $wmsbmt;						# Submit button
	my $wmrset;						# Reset button
	my $lback;						# Back button

	my @puppets;						# List of puppets


	#####################
	# Program area

	# Retreive puppet list
	$database->GetListPuppetYours(\@puppets, $param->{'USER'});

	# Determine values
	# Reply
	if (defined($param->{'REPLY'}))
	{
		# Retreive information
		($from, 
		 $rcpt, 
		 $subj,
	 	 $orig) = $database->DataGet("SELECT
		  MailRcpt,
		  MailFrom,
		  MailSubject,
		  MailContents
		 FROM Mail
		 WHERE MailIDNumber=?", $param->{'REPLY'});

		# Subject line
		unless ($subj =~ /\(RE\)/)
		{
			# Replace subject line
			$subj =~ s/^(.+)/(RE) $subj/s;
		}
	}

	# Normal message
	else
	{
		# Determine values
		$from = (defined($param->{'MFROM'})) ? $param->{'MFROM'} : $puppets[0];
		$rcpt = (defined($param->{'MRCPT'})) ? $param->{'MRCPT'} : '';
		$subj = (defined($param->{'MSUBJ'})) ? $param->{'MSUBJ'} : '';
		$body = (defined($param->{'MBODY'})) ? $param->{'MBODY'} : '';
	}


	# Generate back link
	$url    = $cgi->url(-absolute=>1);
	$inline = $param->EmbedInline(USER=>$param->{'USER'}, CRYPT=>$param->{'CRYPT'});


	# Safety
	$body = (defined($body)) ? $body : '';


	# Widgets
	# Vanity shots
	$database->Pull(\$statement, "SELECT
		  VanityID    AS \"VanityID\",
		  VanityThumb AS \"VanityThumb\"
		 FROM Vanity
		WHERE PuppeteerLogin=?
		ORDER BY VanityID",

	 $param->{'USER'}
	);

	# Check
	if ($res = $statement->fetchrow_hashref())
	{
		# Vanity confirmed in user now to produce
		do
		{
			# Append to ever growing list
			$list .= $tmpl->Pass('TmplVanityList',
				MID     => $res->{'VanityID'},
				LVANITY => $res->{'VanityThumb'},
				WVANITY => $cgi->checkbox(
					-name    => 'VID_' . $res->{'VanityID'},
					-label   => '')
			 );

		} while ($res = $statement->fetchrow_hashref());

		# Generate final template
		$vanity = $tmpl->Pass('TmplVanity',
			LSTVANITY => $list
		 );
	}

	# Finish query
	$statement->finish();


	# Regular widgets
	# Popup menu
	$wchars = (defined($database->{'DEFAULT'})) 
	  ? $cgi->hidden('MFROM', $database->{'DEFAULT'})
	  : $cgi->popup_menu('MFROM', \@puppets, $from);

	# Text fields
	$wmrcpt = $cgi->textfield('MRCPT', $rcpt, 25, 30);
	$wmsubj = $cgi->textfield('MSUBJ', $subj, 25, 250);

	# Text area
	$wmbody = $cgi->textarea(
		-name    => 'MBODY',
		-rows    => 10,
		-columns => 60,
		-wrap    => 'SOFT',
		-default => $body);

	# Buttons
	$wmsbmt = $cgi->submit($mail->{'TxtMailSend'});
	$wmrset = $cgi->reset($mail->{'TxtMailReset'});

	# Links
	$lback    = $url . $inline;


	# Original message as reference
	$orig = (defined($orig))
	  ? $tmpl->Pass('TmplOriginal',
		  MBODY   => $orig,
		  WATTACH => $cgi->checkbox(
			-checked => 'ON',
			-name    => 'ATTACH',
			-label   => ''))
	  : '';


	# Display area
	# Start of form, table
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat(), SEND=>'True'), "\n";

	# Show template
	$tmpl->Show('TmplCompose',
		MORIG  => $orig,
		MVAIN  => $vanity,
		WCHARS => $wchars,
		WMRCPT => $wmrcpt,
		WMSUBJ => $wmsubj,
		WMBODY => $wmbody,
		WMSBMT => $wmsbmt,
		WMRSET => $wmrset,
		LBACK  => $lback
	 );

	# End of form
	print $cgi->end_form(), "\n";
}


#####################
# Mailer Inbox
#
# Shows a listing of all currently received emails, giving an option to delete, view, reply 
# to them in some matter or the other.

sub MailInbox
{
	#####################
	# Data members

	my $database = shift;					# Database Handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $tmpl     = shift;					# Template viewer
	my $mail     = shift;					# Mail hash
	my $action   = shift;					# Action tag

	my $res;						# Results list
	my $statement;						# Query statement

	my $listing;						# Listing of specific messages
	my $internal;						# Listing of internal mail
	my $purge;						# Removing of email
	
	my $color;						# Selected color
	my $unread;						# Unread tag
	my $subject;						# Subject line

	my $icount   = 0;					# Arbitrary counter
	my $mcount   = 0;					# Arbitrary counter

	my $url      = $cgi->url(-absolute=>1);			# Current location

	my $ibio;						# Biographical link
	my $inline;						# Inline parameter list
	my $icomp;						# Composition value
	my $iswitch;						# Inbox to outbox
	my $irefresh;						# Refresh value


	#####################
	# Program area

	# Data gathering
	# Prepare and execute
	$database->Pull(\$statement, "SELECT
		 MailIDNumber  AS \"MailIDNumber\",
		 MailFrom      AS \"MailFrom\",
		 MailFromLogin AS \"MailFromLogin\",
		 MailSubject   AS \"MailSubject\",
		 MailRcptRead  AS \"MailRcptRead\",
		 getDate(MailTimestamp) AS \"MailTimestamp\"
		FROM Mail
		WHERE MailRcptLogin=?
		  AND MailRcptDel='no'
		ORDER BY MailTimestamp DESC, MailFrom",

	 $param->{'USER'}
	);


	# Retreive and display
	if ($res = $statement->fetchrow_hashref())
	{
		# Initialize
		$listing  = '';
		$internal = '';
	
		do
		{
			# Keep or show		 
			# Display information
			unless (defined($param->{"PURGE_$res->{MailIDNumber}"}))
			{
				# Determine values
				# Mail read
				$unread  = ($res->{'MailRcptRead'} eq 'yes')
				  ? $mail->{'TagMailRead'} 
				  : $mail->{'TagMailUnread'};

				# Subject line
				$subject = (length($res->{'MailSubject'}) > 1)
				  ? $res->{'MailSubject'}
				  : $mail->{'TxtSubject'};

				# Generate inline
				$inline = $param->EmbedInline(
					USER  => $param->{'USER'},
					CRYPT => $param->{'CRYPT'},
					MID   => $res->{'MailIDNumber'}
				 );


				# Purging widget
				$purge = $cgi->checkbox(-name=>"PURGE_$res->{MailIDNumber}", -label=>'');

				
				# Normal mail
				if (defined($res->{'MailFrom'}))
				{
					# Increment
					$mcount++;

					# Determine values
					$color   = (($mcount % 2) != 0)
					  ? $mail->{'SetSecColor'}
					  : $mail->{'SetPriColor'};

					# Biographical link
					$ibio = $param->Crypt(
						BCHAR => $res->{'MailFrom'},
						BPOOL => 'system'
					 );


					# Append
					# Use template
					$listing = $listing . $tmpl->Pass('TmplIncoming',
						MCOLOR => $color,
						MMREAD => $unread,
						MMFROM => $res->{'MailFrom'},
						MMSUBJ => $subject,
						MMTIME => $res->{'MailTimestamp'},
						LCHBIO => $mail->{'LnkBio'} . $ibio,
						LMLINK => $url . $inline,
						WDELE  => $purge
					 );
				}

				# Internal messaging
				else
				{
					# Increment
					$icount++;

					# Determine values
					$color   = (($icount % 2) != 0)
					  ? $mail->{'SetSecColor'}
					  : $mail->{'SetPriColor'};

					# Append
					# Use template
					$internal = $internal . $tmpl->Pass('TmplInternal',
						MCOLOR => $color,
						MMREAD => $unread,
						MMSUBJ => $subject,
						MMTIME => $res->{'MailTimestamp'},
						LMLINK => $url . $inline,
						WDELE  => $purge
					 );
				}
			}

			# Delete
			else
			{
				# Delete and ask questions delete
				$database->Quick("UPDATE Mail
					  SET MailRcptDel='yes'
					WHERE MailIDNumber=?
					  AND MailRcptLogin=?",

				 $res->{'MailIDNumber'},
				 $param->{'USER'}
				);
			}

		# All good things must come to an end
		} while ($res = $statement->fetchrow_hashref());

		# End execution
		$statement->finish();
	}

	# Be nice to user
	$listing  = $mail->{'TagMessEmpty'} if ($mcount == 0);
	$internal = $mail->{'TagIntEmpty'}  if ($icount == 0);
	

	# Determine composer values
	$icomp = $param->EmbedInline(
		USER    => $param->{'USER'},
		CRYPT   => $param->{'CRYPT'},
		WRITE   => 'True'
	 );

	# Refresh value
	$irefresh = $param->EmbedInline(
		USER    => $param->{'USER'},
		CRYPT   => $param->{'CRYPT'}
	 );

 	# Switch
	$iswitch = $param->EmbedInline(
		USER    => $param->{'USER'},
		CRYPT   => $param->{'CRYPT'},
		ARCHIVE => 'True'
	 );


	# Action check
	$action = (defined($action)) ? $mail->{'TagMailSent'} : '';


	# Form handling
	print $cgi->start_form();
	print $param->EmbedNormal(USER=>$param->{'USER'}, CRYPT=>$param->{'CRYPT'});

	# Display master template
	$tmpl->Show('TmplInbox',
		LSTMAIL => $listing,
		LSTINT  => $internal,
		MMSENT  => $action,
		MMTOTAL => $mcount,
		MITOTAL => $icount,
		LMCOMP  => $url . $icomp,
		LRLOAD  => $url . $irefresh,
		LSWITCH => $url . $iswitch,
		WDELE   => $cgi->submit($mail->{'TxtMailDelete'})
	 );

	# End form
	print $cgi->end_form(), "\n";
}


#####################
# Mailer Sent Mail
#
# Shows a listing of all sent emails, giving an option to delete, view 
# to them in some matter or the other.

sub MailOutbox
{
	#####################
	# Data members

	my $database = shift;					# Database Handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $tmpl     = shift;					# Template viewer
	my $mail     = shift;					# Mail hash
	my $action   = shift;					# Action tag

	my $res;						# Results list
	my $statement;						# Query statement

	my $listing;						# Listing of specific messages
	my $purge;						# Removing of email
	
	my $color;						# Selected color
	my $undel;						# Not yet removed
	my $unread;						# Unread tag
	my $subject;						# Subject line

	my $icount   = 0;					# Arbitrary counter
	my $mcount   = 0;					# Arbitrary counter

	my $url      = $cgi->url(-absolute=>1);			# Current location

	my $ibio;						# Biographical link
	my $inline;						# Inline parameter list
	my $icomp;						# Composition value
	my $irefresh;						# Refresh value
	my $iswitch;						# Switch from one box to another


	#####################
	# Program area

	# Data gathering
	# Prepare and execute
	$database->Pull(\$statement, "SELECT
		 MailIDNumber AS \"MailIDNumber\",
		 MailRcpt     AS \"MailRcpt\",
		 MailSubject  AS \"MailSubject\",
		 MailRcptRead AS \"MailRcptRead\",
		 MailRcptDel  AS \"MailRcptDel\",
		 MailFromDel  AS \"MailFromRead\",
		 getDate(MailTimestamp) AS \"MailTimestamp\"
		FROM Mail
		WHERE MailFromLogin=?
		  AND MailFromDel='no'
		ORDER BY MailTimestamp DESC, MailFrom",

	 $param->{'USER'}
	);


	# Retreive and display
	if ($res = $statement->fetchrow_hashref())
	{
		# Initialize
		$listing  = '';
	
		do
		{

			# Keep or show		 
			# Display information
			unless (defined($param->{"PURGE_$res->{MailIDNumber}"}))
			{
				# Determine values
				# Mail read
				$unread  = ($res->{'MailRcptRead'} eq 'yes')
				  ? $mail->{'TagMailRead'} 
				  : $mail->{'TagMailUnread'};

				# Deleted
				$undel  = ($res->{'MailRcptDel'} eq 'yes')
				  ? $mail->{'TagMailDel'} 
				  : $mail->{'TagMailUndel'};

				# Subject line
				$subject = (length($res->{'MailSubject'}) > 1)
				  ? $res->{'MailSubject'}
				  : $mail->{'TxtSubject'};

				# Generate inline
				$inline = $param->EmbedInline(
					USER  => $param->{'USER'},
					CRYPT => $param->{'CRYPT'},
					MID   => $res->{'MailIDNumber'}
				 );


				# Purging widget
				$purge = $cgi->checkbox(-name=>"PURGE_$res->{MailIDNumber}", -label=>'');

				
				# Normal mail
				# Increment
				$mcount++;

				# Determine values
				$color   = (($mcount % 2) != 0)
				  ? $mail->{'SetSecColor'}
				  : $mail->{'SetPriColor'};

				# Biographical link
				$ibio = $param->Crypt(
					BCHAR => $res->{'MailRcpt'},
					BPOOL => 'system'
				 );


				# Append
				# Use template
				$listing = $listing . $tmpl->Pass('TmplOutgoing',
					MCOLOR => $color,
					MMDEL  => $undel,
					MMREAD => $unread,
					MMRCPT => $res->{'MailRcpt'},
					MMSUBJ => $subject,
					MMTIME => $res->{'MailTimestamp'},
					LCHBIO => $mail->{'LnkBio'} . $ibio,
					LMLINK => $url . $inline,
					WDELE  => $purge
				 );
			}

			# Delete
			else
			{
				# Delete and ask questions delete
				$database->Quick("UPDATE Mail
					  SET MailFromDel='yes'
					WHERE MailIDNumber=?
					  AND MailFromLogin=?",

				 $res->{'MailIDNumber'},
				 $param->{'USER'}
				);
			}

		# All good things must come to an end
		} while ($res = $statement->fetchrow_hashref());

		# End execution
		$statement->finish();
	}

	# Be nice to user
	$listing  = $mail->{'TagMessEmpty'} if ($mcount == 0);
	

	# Determine composer values
	$icomp = $param->EmbedInline(
		USER    => $param->{'USER'},
		CRYPT   => $param->{'CRYPT'},
		WRITE   => 'True'
	 );

	# Refresh value
	$irefresh = $param->EmbedInline(
		USER    => $param->{'USER'},
		CRYPT   => $param->{'CRYPT'},
		ARCHIVE => 'True'
	 );

	# Switch
	$iswitch = $param->EmbedInline(
		USER  => $param->{'USER'},
		CRYPT => $param->{'CRYPT'}
	 );
	

	# Action check
	$action = (defined($action))
	  ? $mail->{'TagMailSent'}
	  : '';


	# Form handling
	print $cgi->start_form();
	print $param->EmbedNormal(USER=>$param->{'USER'}, CRYPT=>$param->{'CRYPT'}, ARCHIVE=>'True');

	# Display master template
	$tmpl->Show('TmplOutbox',
		LSTMAIL => $listing,
		MMSENT  => $action,
		MMTOTAL => $mcount,
		LMCOMP  => $url . $icomp,
		LRLOAD  => $url . $irefresh,
		LSWITCH => $url . $iswitch,
		WDELE   => $cgi->submit($mail->{'TxtMailDelete'})
	 );

	# End form
	print $cgi->end_form(), "\n";
}


#####################
# Mail Purge
#
# Allows for the purging of vile messages from the list of known messages.   Simple and easy
# to use.

sub MailPurge
{
	#####################
	# Data members

	my $database = shift;					# Database Handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $tmpl     = shift;					# Template listing
	my $mail     = shift;					# Mail hash

	my $id;							# Identification number;
	my $rcpt;						# Recipient
	my $from;						# From tag


	#####################
	# Program area

	# Retreive id
	$id = $param->{'OID'};

	# Parameter handling
	$param->Cleanup('OID');


	# Check the basics
	($rcpt, $from) = $database->DataGet("SELECT
		 MailRcptLogin,
		 MailFromLogin
		FROM Mail
		WHERE MailIDNumber=?", $id);

	
	# Change accordingly
	# Originator
	if ((defined($from))
	 && ($from eq $param->{'USER'}))	
 	{
		# Remove entry
		$database->Quick("UPDATE Mail 
			  SET MailFromDel='yes'
			WHERE MailIDNumber=?
			  AND MailFromLogin=?",
	
		 $id,
		 $param->{'USER'});

		# Call listing
		MailOutbox($database, $cgi, $param, $tmpl, $mail);
	}

	# Recipient
	elsif ((defined($rcpt))
	 && ($rcpt eq $param->{'USER'}))	
 	{
		# Remove entry
		$database->Quick("UPDATE Mail 
			  SET MailRcptDel='yes'
			WHERE MailIDNumber=?
			  AND MailRcptLogin=?",
	
		 $id,
		 $param->{'USER'});

		# Call listing
		MailInbox($database, $cgi, $param, $tmpl, $mail);
	}
}



#####################
# MailSender
#
# Allows for composed message to be sent out to the designated person.

sub MailSender
{
	#####################
	# Data members

	my $database = shift;					# Database Handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $tmpl     = shift;					# Template
	my $mail     = shift;					# Mail hash

	my $statement;						# Ask questions
	my $res;						# Results list
	
	my $filter;						# Mail filter
	my $send;						# Mail handler

	my $from;						# Origin of message
	my $rcpt;						# Intended audience
	my $subj;						# Subject of message
	my $body;						# Body of message

	my $id;							# ID Number of message
	my $notice;						# Notice message

	my $email;						# Email address
	my $ignored;						# Ignored user
	my $liason;						# Liason with user
	my $mailer;						# Mailer settings
	my $name;						# Name of target
	my $target;						# Display of drop down

	my %system;						# Systems hash


	#####################
	# Program area

	# Data retreival
	$from = $param->{'MFROM'};	
	$rcpt = $param->{'MRCPT'};
	$subj = $param->{'MSUBJ'};
	$body = $param->{'MBODY'};

	# Determine puppeteer
	($target)  = $database->DataGetPuppeteerLogin($rcpt);
	($ignored) = $database->DataGetIgnored($target, $rcpt, $param->{'USER'});


	# Determine action
	# Processing
	if ((defined($target))
	 && (!defined($ignored)))
	{
		# Safety
		# Enable filter
		$filter = new Ethereal::Filter();
		
		# Filter for safety
		$body = $filter->SafeHTML($param->{'MBODY'});

		# Inject Gummies
		$body = $filter->Inject($database, $body);

		# Make sure people can read it
		$body =~ s/\n/<BR>/gs;


		# Additional processing
		if ((defined($param->{'REPLY'}))
		 && (defined($param->{'ATTACH'})))
		{
			# Get the basics
			my ($sender, $reply, $date) = $database->DataGet("SELECT
				 MailFrom,
				 MailContents,
				 getDate(MailTimestamp)
				FROM Mail
				WHERE MailIDNumber=?", $param->{'REPLY'});

			# Rebuild the body appropriatly
			$body .= $tmpl->Pass('TmplReply',
				      MFROM  => $sender,
				      MDATE  => $date,
				      MREPLY => $reply
				  );
		}

		# Prevent additional haunts
		$param->Cleanup('REPLY', 'ATTACH');
		
		
		# Retreive full name
		($name)           = $database->DataGetPuppeteerName($target);
		($email, $mailer) = $database->DataGetPuppeteerEmail($target);


		# Make sure this all works
		#eval
		#{
			# Pull ID
			($id)     = $database->DataGetInsert('seqMail');
			

			# This must be first at all costs
			# Insert into database
			$database->Write("INSERT INTO Mail
				(MailIDNumber,
				 MailFrom,
				 MailFromLogin,
				 MailRcpt,
				 MailRcptLogin,
				 MailSubject,
				 MailContents)
				VALUES(?,?,?,?,?,?,?)", 

			 $id,
			 $from,
			 $param->{'USER'},
			 $rcpt,
			 $target,
			 $subj,
			 $body);

			

			# Vanity attachments
			# Prepare to check for need
			$database->Pull(\$statement, "SELECT
				  VanityID AS \"VanityID\"
				FROM Vanity
				WHERE PuppeteerLogin=?",

			 $param->{'USER'}
			);

			# Find hits and append
			while (($res) = $statement->fetchrow())
			{
				# Create new entries
				$database->Write("INSERT INTO MailVanity
					(MailIDNumber,
					 VanityID)
					VALUES (?,?)",

				 $id,
				 $res
				) if (defined($param->{'VID_' . $res}));
			}

			# Finish query
			$statement->finish();

			
		 	# Social networking
			# Check up and see

			# Sender handling
			# Pull necessary information
			($liason) = $database->DataGetLiason($param->{'USER'}, $target);
			
	
			# Sender has no entry entry
			unless (defined($liason))
			{
				# Create a new entry
				# Neutral with owner being initiate
				$database->Write("INSERT INTO Liason
					(LiasonOwner,
					 LiasonTarget)
					VALUES(?,?)",
	
				 $param->{'USER'},
				 $target
				);
			}

			# Sender has an entry and needs to be updated
			elsif ((defined($liason))
			    && ($liason eq 'nil'))
			{
				# Update as necessary
				$database->Write("UPDATE Liason
					SET LiasonSource='owner'
					WHERE LiasonOwner=?
					  AND LiasonTarget=?",

				 $param->{'USER'},
				 $target
				);
			}


			# Target handling
			# Pull necessary information
			($liason) = $database->DataGetLiason($target, $param->{'USER'});
	
			# Recipient has an entry
			unless (defined($liason))
			{
				# Create a new entry
				# Neutral with target being initiate
				$database->Write("INSERT INTO Liason
					(LiasonOwner,
					 LiasonTarget,
					 LiasonSource)
					VALUES(?,?,'target')",
	
				 $target,
				 $param->{'USER'}
				);
			}


			# Sender has an entry and needs to be updated
			elsif ((defined($liason))
			    && ($liason eq 'nil'))
			{
				# Update as necessary
				$database->Write("UPDATE Liason
					SET LiasonSource='target'
					WHERE LiasonOwner=?
					  AND LiasonTarget=?",

				 $target,
				 $param->{'USER'}
				);
			}


			# Commit all changes
			$database->Commit();
			#};

		# Sound the alarm
		#if ($@)
		#{
			# Issue warning
			#warn("Transaction aborted: $@");

			# Undo incomplete changes
			#$database->Rollback();
			#}


		# Additional tasks
	 	# Notification email
	 	if ($mailer eq 'ext')
		{
			# Final preperations
			# Retreive values
			($notice) = $database->DocumentGetMailNotice();
					
			# Build link
			my $url    = $cgi->url();
			my $inline = $param->Crypt(
				MID   => $id,
				MRCPT => $rcpt,
				MFROM => $from,
				MSUBJ => $subj
			 );


			# Compose and send email
			# Link to hash
			$database->GetHashSystem(\%system);

			# Message information
			if (defined($notice))
			{
				# Send mail
				# Initiate instance
				$send = tie(*MAIL, 'Ethereal::Mail', $database);

				# Initial setup
				$send->SetSubject("You have mail from: $from");
				$send->SetSearch(
					AUTHSTRING  => "$url$inline",
					FULLNAME    => $name,
					ORIGIN      => $from,
					RETURNEMAIL => $system{'SetInfoContactAddress'},
					SERVERNAME  => $system{'SetInfoServer'},
					SUBJECT     => $subj
				 );

				# Recipients
				$send->AddTo("$name <$email>");

				# Message
				print MAIL $notice;

				# Close and Send
				close(MAIL);
			}
		}

		# Show listing
		MailOutbox($database, $cgi, $param, $tmpl, $mail, 'SENT');
	}

	# Imporper puppet name
	else
	{
		# Recall composer
		MailComposer($database, $cgi, $param, $tmpl, $mail);
	}
}


#####################
# MailViewer
#
# Allows for the viewing of mail to be sent out to the designated person. This
# will allow the puppeteer to reply if necessary as to make life simpler.

sub MailViewer
{
	#####################
	# Data members

	my $database = shift;					# Database Handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $tmpl     = shift;					# Template instance
	my $mail     = shift;					# Mail hash

	my $statement;						# Database statement
	my $res;						# Results list

	my $list     = '';					# Listing of vanities
	my $vanity   = '';					# Vanity template
	
	my $body;						# Body of message
	my $from;						# Origin of message
	my $froml;						# Login origin of message
	my $rcpt;						# Intended audience
	my $rcptl;						# Login intended audience
	my $subj;						# Subject of message
	my $time;						# Time of posting

	my $inline;						# Inline parameter
	my $url;						# Self reference

	my $id;							# Identification number;


	#####################
	# Program area

	# Identification number
	$id = $param->{'MID'};
	$param->Cleanup('MID');

	# Assign OID
	$param->{'OID'} = $id;

	# Retreive information
	($from, 
	 $froml,
	 $rcpt, 
	 $rcptl,
	 $subj, 
	 $time, 
	 $body) = $database->DataGet("SELECT
		  MailFrom,
		  MailFromLogin,
		  MailRcpt,
		  MailRcptLogin,
		  MailSubject,
		  getDate(MailTimestamp),
		  MailContents
		 FROM Mail
		 WHERE MailIDNumber=?", $id); 


	# Only complete if intended target
	if (($froml eq $param->{'USER'})
	 or ($rcptl eq $param->{'USER'}))
	{
		# Generate back
		$url    = $cgi->url(-absolute=>1);
		$inline = $param->EmbedInline(
			USER  => $param->{'USER'},
			CRYPT => $param->{'CRYPT'}
		 );

		# Display area
		# Start of form
		print $cgi->start_form();
		print $param->EmbedNormal($param->Flat(),
			REPLY => $id,
			WRITE => 'True'), "\n";

		# Template select
		# Normal message
		if (defined($from))
		{		
			# In the quest to get laid
			# Check for vanity
			$database->Pull(\$statement, "SELECT
				  v.VanityID     AS \"VanityID\",
				  v.VanityPath   AS \"VanityPath\",
				  v.VanityWidth  AS \"VanityWidth\",
				  v.VanityHeight AS \"VanityHeight\"
				FROM Vanity v, MailVanity m
				WHERE v.VanityID=m.VanityID
				  AND m.MailIDNumber=?
				ORDER BY v.VanityID", $id);

			# Time to subject users to vanity
			if ($res = $statement->fetchrow_hashref())
			{

				# Loop and pull
				do
				{
					# Append to every growing list
					$list .= $tmpl->Pass('TmplAttachList',
						LPATH   => $res->{'VanityPath'},
						MID     => $res->{'VanityID'},
				   		MWIDTH  => $res->{'VanityWidth'},
						MHEIGHT => $res->{'VanityHeight'}
					 );

				} while ($res = $statement->fetchrow_hashref());

				# Generate for viewing 'pleasure'
				$vanity = $tmpl->Pass('TmplAttach',
					LSTATTACH => $list
				 );
			}

			# Complete order
			$statement->finish();

			
			# Message display template
			$tmpl->Show('TmplViewer',
				MVAIN  => $vanity,
				MMFROM => $from,
				MMRCPT => $rcpt,
				MMSUBJ => $subj,
				MMBODY => $body,
				MMTIME => $time,
				WMREPL => $cgi->submit($mail->{'TxtMailReply'}),
				WMDELE => $cgi->submit($mail->{'TxtMailDelete'}),
				LBACK  => $url . $inline
			 );
		}

		# Internal messaging lack certain fields
		else
		{
			# Internal display template
			$tmpl->Show('TmplViewerInt',
				MMSUBJ => $subj,
				MMBODY => $body,
				MMTIME => $time,
				WMDELE => $cgi->submit($mail->{'TxtMailDelete'}),
				LBACK  => $url . $inline
			 );
		}

		# Form bottom
		print $cgi->end_form(), "\n";


		# Is update necessary
		if ($rcptl eq $param->{'USER'})
		{
			# Update information to mark read
			$database->Quick("UPDATE Mail
				SET MailRcptRead='yes'
				WHERE MailIDNumber=?",
	
			 $id
			);
		}
	}

	# Invalid component
	else
	{
		# Call listing
		MailInbox($database, $cgi, $param, $tmpl, $mail);
	}
}
